----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    19:41:03 12/25/2006 
-- Design Name: 
-- Module Name:    UpScan1500 - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity UpScan1500 is
  Port ( CLK_14M : in  std_logic;
         HSYNC   : in  std_logic;
			RGBIN   : in  std_logic_vector(2 downto 0);
			TXTIN	: in  std_logic;
			RGBOUT  : out std_logic_vector(5 downto 0);
			ADDR    : out std_logic_vector(8 downto 0);
			DATA    : inout std_logic_vector(7 downto 0);
			WE      : out std_logic;
			OE      : out std_logic;
--			RESET   : in std_logic;		-- DEBUG --
			MODE64	: in std_logic;
			WHITE	: in std_logic;
			HSYNC31 : out std_logic );
			attribute pin_assign : string;
			attribute pin_assign of ADDR : signal is "44,43,42,40,39,38,37,36,35";
			attribute pin_assign of DATA : signal is "34,33,29,28,27,26,25,24";
			attribute pin_assign of RGBIN : signal is "3,2,1";
			attribute pin_assign of TXTIN : signal is "4";
			attribute pin_assign of RGBOUT : signal is "14,13,9,8,7,6";
			attribute pin_assign of CLK_14M : signal is "5";
			attribute pin_assign of HSYNC : signal is "11";
			attribute pin_assign of HSYNC31 : signal is "12";
			attribute pin_assign of WE : signal is "19";
			attribute pin_assign of OE : signal is "20";
			attribute pin_assign of MODE64 : signal is "22";
			attribute pin_assign of WHITE : signal is "18";
end UpScan1500;

architecture Behavioral of UpScan1500 is
  signal RDCNT			: std_logic_vector(8 downto 0);
  signal WRCNT			: std_logic_vector(7 downto 0);
  signal HSY_SMPL		: std_logic;
  signal RGBIN_SMPL	: std_logic_vector(7 downto 0);
  signal RGBOUT_SMPL	: std_logic_vector(7 downto 0);
  signal RSWAP			: std_logic;

begin

-- ** SRAM OE/WE **
	OE <= CLK_14M or RDCNT(0);
--Test--	WE <= CLK_14M or (RDCNT(0) nand RDCNT(1));
	WE <= CLK_14M or ((not RDCNT(1)) nand RDCNT(0));

-- ** SRAM Address/Data **
  process( RDCNT, WRCNT, RSWAP, RGBIN_SMPL, DATA ) begin
--Test--    if RDCNT(1 downto 0) = "11" then
    if RDCNT(1 downto 0) = "01" then
		ADDR <= (not RSWAP) & WRCNT;
		DATA <= RGBIN_SMPL(7 downto 0);
	 else
		ADDR <= RSWAP & RDCNT(8 downto 1);
		DATA <= ( others=>'Z' );
	 end if;
  end process;

-- ** RGB Input **
	process( RDCNT(0) ) begin
		if RDCNT(0) = '1' and RDCNT(0)'event then
			RGBIN_SMPL(7 downto 4) <= RGBIN_SMPL(3 downto 0);
			RGBIN_SMPL(3 downto 0) <= TXTIN & RGBIN;
		end if;
	end process;

-- ** RGB Output **
	process( RGBOUT_SMPL, MODE64, WHITE, RDCNT ) begin
		if ( MODE64 = '0' ) then
			if ( RDCNT(0) = '1' ) then
				RGBOUT(5) <= RGBOUT_SMPL(6) or WHITE;		--Green 1
				RGBOUT(4) <= RGBOUT_SMPL(5) or WHITE;		--Red   1
				RGBOUT(3) <= RGBOUT_SMPL(4) and (not WHITE);	--Blue  1
				RGBOUT(2) <= RGBOUT_SMPL(6) or WHITE;		--Green 0
				RGBOUT(1) <= RGBOUT_SMPL(5) and (not WHITE);	--Red   0
				RGBOUT(0) <= RGBOUT_SMPL(4) or WHITE;		--Blue  0
			else
				RGBOUT(5) <= RGBOUT_SMPL(2) or WHITE;		--Green 1
				RGBOUT(4) <= RGBOUT_SMPL(1) or WHITE;		--Red   1
				RGBOUT(3) <= RGBOUT_SMPL(0) and (not WHITE);	--Blue  1
				RGBOUT(2) <= RGBOUT_SMPL(2) or WHITE;		--Green 0
				RGBOUT(1) <= RGBOUT_SMPL(1) and (not WHITE);	--Red   0
				RGBOUT(0) <= RGBOUT_SMPL(0) or WHITE;		--Blue  0
			end if;
		else
			if ( RDCNT(0) = '1' ) then
				RGBOUT(5) <= (RGBOUT_SMPL(6) or RGBOUT_SMPL(7));	--Green 1
				RGBOUT(4) <= (RGBOUT_SMPL(5) or RGBOUT_SMPL(7));	--Red   1
				RGBOUT(3) <= (RGBOUT_SMPL(4) or RGBOUT_SMPL(7));	--Blue  1
				RGBOUT(2) <= (RGBOUT_SMPL(2) or RGBOUT_SMPL(7));	--Green 0
				RGBOUT(1) <= (RGBOUT_SMPL(1) or RGBOUT_SMPL(7));	--Red   0
				RGBOUT(0) <= (RGBOUT_SMPL(0) or RGBOUT_SMPL(7));	--Blue  0
			else
				RGBOUT(5) <= (RGBOUT_SMPL(6) or RGBOUT_SMPL(3));	--Green 1
				RGBOUT(4) <= (RGBOUT_SMPL(5) or RGBOUT_SMPL(3));	--Red   1
				RGBOUT(3) <= (RGBOUT_SMPL(4) or RGBOUT_SMPL(3));	--Blue  1
				RGBOUT(2) <= (RGBOUT_SMPL(2) or RGBOUT_SMPL(3));	--Green 0
				RGBOUT(1) <= (RGBOUT_SMPL(1) or RGBOUT_SMPL(3));	--Red   0
				RGBOUT(0) <= (RGBOUT_SMPL(0) or RGBOUT_SMPL(3));	--Blue  0
			end if;
		end if;
	end process;
	process( CLK_14M ) begin
		if CLK_14M = '1' and CLK_14M'event then
			if RDCNT(0) = '0' then
				RGBOUT_SMPL <= DATA;
			end if;
		end if;
	end process;

-- ** Read counter **
  process( CLK_14M ) begin
    if CLK_14M = '1' and CLK_14M'event then
	   if (HSY_SMPL = '1' and HSYNC = '0') or RDCNT = 455 then
		  RDCNT <= "000000000";
		else
		  RDCNT <= RDCNT + '1';
		end if;
	 end if;
  end process;

-- ** Write counter **
  process( CLK_14M ) begin
    if CLK_14M = '1' and CLK_14M'event then
	   if (HSY_SMPL = '1' and HSYNC = '0') then
		  WRCNT <= "00000000";
		else
		  if RDCNT(1 downto 0) = "11" then
		    WRCNT <= WRCNT + '1';
		  end if;
		end if;
	 end if;
  end process;

-- ** HSYNC sample **
  process( CLK_14M ) begin
    if CLK_14M = '0' and CLK_14M'event then
		HSY_SMPL <= HSYNC;
	 end if;
  end process;

-- ** HSYNC 31kHz output **
  HSYNC31 <= RDCNT(8) or RDCNT(7) or RDCNT(6) or RDCNT(5);

-- ** Raster Buffer Swapping **
  process( CLK_14M ) begin
    if CLK_14M = '1' and CLK_14M'event then
--		if RESET = '0' then RSWAP <= '0'; elsif HSY_SMPL = "10" then --DEBUG--
	   if (HSY_SMPL = '1' and HSYNC = '0') then
		  RSWAP <= not RSWAP;
		end if;
	 end if;
  end process;

end Behavioral;

