----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    19:41:03 12/25/2006 
-- Design Name: 
-- Module Name:    UpScan700 - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity UpScan700 is
 Port (	CLK_14M : in  std_logic;
        HSYNC   : in  std_logic;
	RGBIN   : in  std_logic_vector(2 downto 0);
	RGBOUT  : out std_logic_vector(2 downto 0);
	ADDR    : out std_logic_vector(8 downto 0);
	DATA    : inout std_logic_vector(7 downto 0);
	WE      : out std_logic;
	OE      : out std_logic;
	HSYNC31 : out std_logic );
	attribute pin_assign : string;
	attribute pin_assign of ADDR	: signal is "44,43,42,40,39,38,37,36,35";
	attribute pin_assign of DATA	: signal is "34,33,29,28,27,26,25,24";
	attribute pin_assign of RGBIN	: signal is "3,2,1";
	attribute pin_assign of RGBOUT	: signal is "8,7,6";
	attribute pin_assign of CLK_14M	: signal is "5";
	attribute pin_assign of HSYNC	: signal is "11";
	attribute pin_assign of HSYNC31	: signal is "12";
	attribute pin_assign of WE	: signal is "19";
	attribute pin_assign of OE	: signal is "20";
end UpScan700;

architecture Behavioral of UpScan700 is
	signal RDCNT		: std_logic_vector(8 downto 0);
	signal WRCNT		: std_logic_vector(7 downto 0);
	signal HSY_SMPL		: std_logic;
	signal RGBIN_SMPL	: std_logic_vector(5 downto 0);
	signal RGBOUT_SMPL	: std_logic_vector(5 downto 0);
	signal RSWAP		: std_logic;

begin

-- ** SRAM OE/WE **
	OE <= CLK_14M or RDCNT(0);
	WE <= CLK_14M or (RDCNT(0) nand RDCNT(1));

-- ** SRAM Address/Data **
  process( RDCNT, WRCNT, RSWAP, RGBIN_SMPL, DATA ) begin
    if RDCNT(1 downto 0) = "11" then
	ADDR <= (not RSWAP) & WRCNT;
	DATA <= '0' & RGBIN_SMPL(5 downto 3) & '0' & RGBIN_SMPL(2 downto 0);
     else
	ADDR <= RSWAP & RDCNT(8 downto 1);
	DATA <= ( others=>'Z' );
    end if;
  end process;

-- ** RGB Input **
	process( RDCNT(0) ) begin
		if RDCNT(0) = '1' and RDCNT(0)'event then
			RGBIN_SMPL(5 downto 3) <= RGBIN_SMPL(2 downto 0);
			RGBIN_SMPL(2 downto 0) <= RGBIN;
		end if;
	end process;

-- ** RGB Output **
	RGBOUT <= RGBOUT_SMPL(5 downto 3);
	process( CLK_14M ) begin
		if CLK_14M = '1' and CLK_14M'event then
			if RDCNT(0) = '0' then
				RGBOUT_SMPL <= DATA(6 downto 4) & DATA(2 downto 0);
			else
				RGBOUT_SMPL(5 downto 3) <= RGBOUT_SMPL(2 downto 0);
			end if;
		end if;
	end process;

-- ** Read counter **
  process( CLK_14M ) begin
    if CLK_14M = '1' and CLK_14M'event then
	if (HSY_SMPL = '1' and HSYNC = '0') or RDCNT = 455 then
	    RDCNT <= "000000000";
	else
	    RDCNT <= RDCNT + '1';
	end if;
    end if;
  end process;

-- ** Write counter **
  process( CLK_14M ) begin
    if CLK_14M = '1' and CLK_14M'event then
	if (HSY_SMPL = '1' and HSYNC = '0') then
	    WRCNT <= "00000000";
	else
	    if RDCNT(1 downto 0) = "11" then
		WRCNT <= WRCNT + '1';
	    end if;
	end if;
     end if;
  end process;

-- ** HSYNC sample **
  process( CLK_14M ) begin
    if CLK_14M = '0' and CLK_14M'event then
	HSY_SMPL <= HSYNC;
     end if;
  end process;

-- ** HSYNC 31kHz output **
  HSYNC31 <= RDCNT(8) or RDCNT(7) or RDCNT(6) or RDCNT(5);

-- ** Raster Buffer Swapping **
  process( CLK_14M ) begin
    if CLK_14M = '1' and CLK_14M'event then
	if (HSY_SMPL = '1' and HSYNC = '0') then
	    RSWAP <= not RSWAP;
	end if;
    end if;
  end process;

end Behavioral;

